# flake8: noqa
from keplemon.time import Epoch, TimeSpan
from keplemon.elements import HorizonState, CartesianVector, TopocentricElements
from keplemon.enums import ReferenceFrame

class FieldOfViewCandidate:
    satellite_id: str
    """ID of the candidate satellite"""

    direction: TopocentricElements
    """Measured direction to the candidate satellite in the sensor's topocentric frame"""

class FieldOfViewReport:
    epoch: Epoch
    """UTC epoch of the field of view report"""

    sensor_position: CartesianVector
    """TEME position of the sensor in the observatory's topocentric frame in **_kilometers_**"""

    sensor_direction: TopocentricElements
    """Direction of the sensor in the observatory's topocentric frame"""

    fov_angle: float
    """Field of view angle of the sensor in **_degrees_**"""

    candidates: list[FieldOfViewCandidate]
    """List of candidate satellites within the field of view"""

    reference_frame: ReferenceFrame
    """Reference frame of the output direction elements"""

class CloseApproach:
    epoch: Epoch
    """UTC epoch of the close approach"""

    primary_id: str
    """Satellite ID of the primary body in the close approach"""

    secondary_id: str
    """Satellite ID of the secondary body in the close approach"""

    distance: float
    """Distance between the two bodies in **_kilometers_**"""

class CloseApproachReport:
    """
    Args:
        start: CA screening start time
        end: CA screening end time
        distance_threshold: Distance threshold for CA screening in **_kilometers_**
    """

    close_approaches: list[CloseApproach]
    """List of close approaches found during the screening"""

    distance_threshold: float
    def __init__(self, start: Epoch, end: Epoch, distance_threshold: float) -> None: ...

class HorizonAccess:

    satellite_id: str
    """ID of the satellite for which the access is calculated"""

    observatory_id: str
    """ID of the observatory for which the access is calculated"""

    start: HorizonState
    """State of the satellite at the start of the access period"""

    end: HorizonState
    """State of the satellite at the end of the access period"""

class HorizonAccessReport:
    """
    Args:
        start: UTC epoch of the start of the access report
        end: UTC epoch of the end of the access report
        min_elevation: Minimum elevation angle for access in **_degrees_**
        min_duration: Minimum duration of access
    """

    accesses: list[HorizonAccess]
    """List of horizon accesses found during the screening"""

    elevation_threshold: float
    """Minimum elevation angle for access in **_degrees_**"""

    start: Epoch
    """UTC epoch of the start of the access report"""

    end: Epoch
    """UTC epoch of the end of the access report"""

    duration_threshold: TimeSpan
    """Minimum duration of a valid access"""

    def __init__(
        self,
        start: Epoch,
        end: Epoch,
        min_elevation: float,
        min_duration: TimeSpan,
    ) -> None: ...



# flake8: noqa
from typing import Optional

from keplemon.elements import (
    TLE,
    CartesianState,
    Ephemeris,
    KeplerianState,
    GeodeticPosition,
    OrbitPlotData,
    TopocentricElements,
    RelativeState,
    BoreToBodyAngles,
)
from keplemon.catalogs import TLECatalog
from keplemon.time import Epoch, TimeSpan
from keplemon.events import CloseApproach, CloseApproachReport, HorizonAccessReport, FieldOfViewReport
from keplemon.propagation import ForceProperties
from keplemon.enums import ReferenceFrame

class Satellite:

    id: str
    """Unique identifier for the satellite."""

    norad_id: int
    """Number corresponding to the satellite's NORAD catalog ID.
    """

    force_properties: ForceProperties
    """Force properties of the satellite used for propagation"""

    name: Optional[str]
    """Human-readable name of the satellite"""

    keplerian_state: Optional[KeplerianState]
    """Keplerian state of the satellite at the epoch of the TLE, if available"""

    geodetic_position: Optional[GeodeticPosition]
    """Geodetic position of the satellite at the epoch of the TLE, if available"""

    def __init__(self) -> None: ...
    @classmethod
    def from_tle(cls, tle: TLE) -> Satellite:
        """
        Instantiate a Satellite from a legacy TLE

        Args:
            tle: Two-line element set for the satellite
        """
        ...

    def get_close_approach(
        self,
        other: Satellite,
        start: Epoch,
        end: Epoch,
        distance_threshold: float,
    ) -> Optional[CloseApproach]: ...
    def get_ephemeris(
        self,
        start: Epoch,
        end: Epoch,
        step: TimeSpan,
    ) -> Ephemeris: ...
    def get_state_at_epoch(self, epoch: Epoch) -> CartesianState: ...
    def to_tle(self) -> Optional[TLE]:
        """
        Returns:
            Satellite as a two-line element set or None if no state is loaded

        """
        ...

    def get_relative_state_at_epoch(self, other: Satellite, epoch: Epoch) -> Optional[RelativeState]:
        """
        Calculate the relative state between this satellite and another satellite at a given epoch.

        Args:
            other: Secondary satellite to calculate the relative state against
            epoch: UTC epoch at which the relative state will be calculated
        """
        ...

    def get_body_angles_at_epoch(self, other: Satellite, epoch: Epoch) -> Optional[BoreToBodyAngles]:
        """
        Calculate the bore-to-body angles between this satellite and another satellite at a given epoch.

        Args:
            other: Secondary satellite to calculate the bore-to-body angles against
            epoch: UTC epoch at which the bore-to-body angles will be calculated
        """
        ...

    def get_plot_data(self, start: Epoch, end: Epoch, step: TimeSpan) -> Optional[OrbitPlotData]: ...
    def get_observatory_access_report(
        self,
        observatories: list[Observatory],
        start: Epoch,
        end: Epoch,
        min_el: float,
        min_duration: TimeSpan,
    ) -> Optional[HorizonAccessReport]:
        """
        Calculate horizon access from multiple observatories to this satellite.

        Args:
            observatories: List of observatories to check for horizon access
            start: UTC epoch of the start of the report
            end: UTC epoch of the end of the report
            min_el: Minimum elevation angle in **_degrees_**
            min_duration: Minimum duration of access

        Returns:
            Horizon access report containing accesses from all observatories to the satellite,
            or None if the satellite ephemeris cannot be generated
        """
        ...

class Constellation:

    count: int
    """Number of satellites in the constellation"""

    name: Optional[str]
    """Human-readable name of the constellation"""

    def __init__(self) -> None: ...
    def get_plot_data(self, start: Epoch, end: Epoch, step: TimeSpan) -> dict[str, OrbitPlotData]: ...
    @classmethod
    def from_tle_catalog(cls, tle_catalog: TLECatalog) -> Constellation:
        """
        Instantiate a Constellation from a TLE catalog

        Args:
            tle_catalog: TLE catalog for the constellation
        """
        ...

    def get_states_at_epoch(self, epoch: Epoch) -> dict[int, CartesianState]:
        """
        Args:
            epoch: UTC epoch at which the states will be calculated

        Returns:
            (satellite_id, state) dictionary for the constellation at the given epoch
        """
        ...

    def get_ephemeris(
        self,
        start: Epoch,
        end: Epoch,
        step: TimeSpan,
    ) -> dict[str, Ephemeris]:
        """
        Args:
            start: UTC epoch of the start of the ephemeris
            end: UTC epoch of the end of the ephemeris
            step: Time step for the ephemeris

        Returns:
            (satellite_id, ephemeris) dictionary for the constellation
        """
        ...

    def get_ca_report_vs_one(
        self,
        other: Satellite,
        start: Epoch,
        end: Epoch,
        distance_threshold: float,
    ) -> CloseApproachReport:
        """
        Calculate close approaches between the constellation and a given satellite.

        Args:
            other: Satellite to compare against
            start: UTC epoch of the start of the close approach report
            end: UTC epoch of the end of the close approach report
            distance_threshold: Distance threshold for close approach screening in **_kilometers_**

        Returns:
            Close approach report for the constellation vs. the given satellite
        """
        ...

    def get_ca_report_vs_many(
        self,
        start: Epoch,
        end: Epoch,
        distance_threshold: float,
    ) -> CloseApproachReport:
        """
        Calculate close approaches among satellites in the calling constellation.

        !!! warning
            This is a long-running operation when the constellation is large.

        Args:
            start: UTC epoch of the start of the close approach report
            end: UTC epoch of the end of the close approach report
            distance_threshold: Distance threshold for close approach screening in **_kilometers_**

        Returns:
            Close approach report for the constellation vs. all other satellites
        """
        ...

    def __getitem__(self, satellite_id: str) -> Satellite: ...
    def __setitem__(self, satellite_id: str, sat: Satellite) -> None: ...
    def get_horizon_access_report(
        self,
        site: Observatory,
        start: Epoch,
        end: Epoch,
        min_el: float,
        min_duration: TimeSpan,
    ) -> HorizonAccessReport:
        """
        Calculate horizon access to a given observatory.

        Args:
            site: Observatory to check for horizon access
            start: UTC epoch of the start of the report
            end: UTC epoch of the end of the report
            min_el: Minimum elevation angle in **_degrees_**
            min_duration: Minimum duration of access

        Returns:
            Horizon access report for the constellation from the observatory
        """
        ...

class Sensor:
    """
    Args:
        name: Identifier of the sensor
        angular_noise: Angular noise in **_degrees_**
    """

    id: str
    """Unique identifier for the sensor."""
    name: Optional[str]
    angular_noise: float
    range_noise: Optional[float]
    """Range noise in **_kilometers_**"""

    range_rate_noise: Optional[float]
    """Range rate noise in **_kilometers per second_**"""

    angular_rate_noise: Optional[float]
    """Angular rate noise in **_degrees per second_**"""
    def __init__(self, angular_noise: float) -> None: ...

class Observatory:
    """
    Args:
        latitude: Latitude in **_degrees_**
        longitude: Longitude in **_degrees_**
        altitude: Altitude in **_kilometers_**
    """

    name: str
    id: str
    """Unique identifier for the observatory."""
    latitude: float
    longitude: float
    altitude: float
    sensors: list[Sensor]
    """List of sensors at the observatory"""
    def __init__(
        self,
        latitude: float,
        longitude: float,
        altitude: float,
    ) -> None: ...
    def get_state_at_epoch(self, epoch: Epoch) -> CartesianState:
        """
        Args:
            epoch: UTC epoch of the state

        Returns:
            TEME Cartesian state of the observatory in **_kilometers_** and **_kilometers per second_**
        """
        ...

    @classmethod
    def from_cartesian_state(cls, state: CartesianState) -> Observatory:
        """
        Create an observatory from a Cartesian state.

        Args:
            state: Cartesian state of the observatory

        """
        ...

    def get_theta(self, epoch: Epoch) -> float:
        """
        Calculate the Greenwich angle plus the observatory longitude at a given epoch.

        Args:
            epoch: UTC epoch for the calculation

        Returns:
            Greenwich angle plus the observatory longitude in **_radians_**
        """
        ...

    def get_horizon_access_report(
        self,
        satellite: Satellite,
        start: Epoch,
        end: Epoch,
        min_el: float,
        min_duration: TimeSpan,
    ) -> HorizonAccessReport:
        """
        Calculate horizon access for a satellite from the observatory.

        Args:
            satellite: Satellite to check for horizon access
            start: UTC epoch of the start of the report
            end: UTC epoch of the end of the report
            min_el: Minimum elevation angle in **_degrees_**
            min_duration: Minimum duration of access in **_seconds_**

        Returns:
            Horizon access report for the satellite from the observatory
        """
        ...

    def get_field_of_view_report(
        self,
        epoch: Epoch,
        sensor_direction: TopocentricElements,
        angular_threshold: float,
        sats: Constellation,
        reference_frame: ReferenceFrame,
    ) -> FieldOfViewReport:
        """
        Calculate satellites in the field of view from a given time and direction.

        Args:
            epoch: UTC epoch of the report
            sensor_direction: Topocentric direction the sensor is pointing
            angular_threshold: Angular threshold in **_degrees_**
            sats: Constellation of satellites to check for being in the field of view
            reference_frame: Reference frame of the output direction elements
        """
        ...

    def get_topocentric_to_satellite(
        self,
        epoch: Epoch,
        sat: Satellite,
        reference_frame: ReferenceFrame,
    ) -> TopocentricElements:
        """
        Get the topocentric elements of a satellite as seen from the observatory.
        Args:
            epoch: UTC epoch of the observation
            sat: Satellite to observe
            reference_frame: Reference frame of the output direction elements
        """
        ...



from keplemon._keplemon.events import (  # type: ignore
    CloseApproach,
    CloseApproachReport,
    HorizonAccess,
    HorizonAccessReport,
    FieldOfViewCandidate,
    FieldOfViewReport,
)

__all__ = [
    "CloseApproach",
    "CloseApproachReport",
    "HorizonAccess",
    "HorizonAccessReport",
    "FieldOfViewCandidate",
    "FieldOfViewReport",
]



from keplemon._keplemon.estimation import (  # type: ignore
    Observation,
    ObservationResidual,
    BatchLeastSquares,
    Covariance,
    ObservationAssociation,
)

__all__ = ["Observation", "ObservationResidual", "BatchLeastSquares", "Covariance", "ObservationAssociation"]



# flake8: noqa
from __future__ import annotations
from typing import Optional

from keplemon.time import Epoch
from keplemon.enums import Classification, KeplerianType, ReferenceFrame
from keplemon.events import CloseApproach
from keplemon.bodies import Observatory

class RelativeState:
    epoch: Epoch
    position: CartesianVector
    velocity: CartesianVector
    origin_satellite_id: str
    secondary_satellite_id: str

class BoreToBodyAngles:
    earth_angle: float
    sun_angle: float
    moon_angle: float

class OrbitPlotState:
    epoch: Epoch
    latitude: float
    longitude: float
    altitude: float
    semi_major_axis: float
    eccentricity: float
    inclination: float
    raan: float
    radius: float
    apogee_radius: float
    perigee_radius: float

class OrbitPlotData:
    satellite_id: str
    epochs: list[str]
    semi_major_axes: list[float]
    eccentricities: list[float]
    inclinations: list[float]
    raans: list[float]
    radii: list[float]
    apogee_radii: list[float]
    perigee_radii: list[float]
    latitudes: list[float]
    longitudes: list[float]
    altitudes: list[float]

class GeodeticPosition:
    """
    Args:
        latitude: Latitude in **_degrees_**
        longitude: Longitude in **_degrees_**
        altitude: Altitude in **_kilometers_**
    """

    latitude: float
    """Latitude in **_degrees_**"""

    longitude: float
    """Longitude in **_degrees_**"""

    altitude: float
    """Altitude in **_kilometers_**"""

    def __init__(self, latitude: float, longitude: float, altitude: float) -> None: ...

class HorizonElements:
    """
    Args:
        range: Range in **_kilometers_**
        az: Azimuth in **_degrees_**
        el: Elevation in **_degrees_**
        range_rate: Range rate in **_kilometers per second_**
        az_rate: Azimuth rate in **_degrees per second_**
        el_rate: Elevation rate in **_degrees per second_**
    """

    range: Optional[float]
    azimuth: float
    elevation: float
    range_rate: Optional[float]
    azimuth_rate: Optional[float]
    elevation_rate: Optional[float]

    def __init__(
        self,
        azimuth: float,
        elevation: float,
    ) -> None: ...

class HorizonState:
    """
    Args:
        epoch: UTC epoch of the state
        elements: HorizonElements of the state
    """

    epoch: Epoch
    """UTC epoch of the state"""

    elements: HorizonElements
    """Horizon elements of the state"""

    range: Optional[float]
    """Range in **_kilometers_**"""

    azimuth: float
    """Azimuth in **_degrees_**"""

    elevation: float
    """Elevation in **_degrees_**"""

    range_rate: Optional[float]
    """Range rate in **_kilometers per second_**"""

    azimuth_rate: Optional[float]
    """Azimuth rate in **_degrees per second_**"""

    elevation_rate: Optional[float]
    """Elevation rate in **_degrees per second_**"""

    def __init__(self, epoch: Epoch, elements: HorizonElements) -> None: ...
    @classmethod
    def from_topocentric_state(cls, state: TopocentricState, observer: Observatory) -> HorizonState: ...

class KeplerianElements:
    """
    Args:
        semi_major_axis: Average distance from the central body in **_kilometers_**
        eccentricity: Eccentricity of the orbit
        inclination: Inclination of the orbit in **_degrees_**
        raan: Right Ascension of Ascending Node in **_degrees_**
        argument_of_perigee: Argument of Perigee in **_degrees_**
        mean_anomaly: Mean Anomaly in **_degrees_**
    """

    semi_major_axis: float
    eccentricity: float
    inclination: float
    raan: float
    argument_of_perigee: float
    mean_anomaly: float

    def __init__(
        self,
        semi_major_axis: float,
        eccentricity: float,
        inclination: float,
        raan: float,
        argument_of_perigee: float,
        mean_anomaly: float,
    ) -> None: ...

class EquinoctialElements:
    """
    Args:
        a_f: Equinoctial element a_f
        a_g: Equinoctial element a_g
        chi: Equinoctial element chi
        psi: Equinoctial element psi
        mean_longitude: Mean longitude in **_degrees_**
        mean_motion: Mean motion in **_revolutions per day_**
    """

    a_f: float
    a_g: float
    chi: float
    psi: float
    mean_longitude: float
    mean_motion: float

    def __init__(
        self,
        a_f: float,
        a_g: float,
        chi: float,
        psi: float,
        mean_longitude: float,
        mean_motion: float,
    ) -> None: ...

    def to_keplerian(self) -> KeplerianElements: ...

class TLE:

    norad_id: int
    """NORAD catalog ID of the satellite"""

    satellite_id: str
    """"""

    name: str
    """"""

    inclination: float
    """Inclination of the orbit in **_degrees_**"""

    eccentricity: float
    """"""

    raan: float
    """Right Ascension of Ascending Node in **_degrees_**"""

    argument_of_perigee: float
    """Argument of Perigee in **_degrees_**"""

    mean_anomaly: float
    """Mean Anomaly in **_degrees_**"""

    mean_motion: float
    """Mean motion in **_revolutions per day_**"""

    type: KeplerianType
    """"""

    b_star: float
    """"""

    mean_motion_dot: float
    """"""

    mean_motion_dot_dot: float
    """"""

    agom: float
    """"""

    b_term: float
    """"""

    epoch: Epoch
    """UTC epoch of the state"""

    classification: Classification
    """"""

    designator: str
    """8-character identifier of the satellite"""

    cartesian_state: CartesianState
    """TEME cartesian state of the TLE at epoch"""

    semi_major_axis: float
    """Average distance from the central body in **_kilometers_**
    
    !!! note
        This is always calculated using Brouwer mean motion and will differ slightly from Kozai-computed SMA.
    """

    apoapsis: float
    """Apoapsis radius in **_kilometers_**"""

    periapsis: float
    """Periapsis radius in **_kilometers_**"""

    @classmethod
    def from_lines(cls, line_1: str, line_2: str, line_3: Optional[str] = None) -> TLE:
        """
        Create a TLE object using strings in 2 or 3 line format
        """
        ...

    @property
    def lines(self) -> tuple[str, str]:
        """
        !!! note
            If the TLE was created in the 3LE format, only lines 2 and 3 will be returned.  The name must be accessed
            using the `name` property.

        Returns:
            Tuple of strings in 2 line format
        """
        ...

    def get_state_at_epoch(self, epoch: Epoch) -> CartesianState: ...

class SphericalVector:
    """
    !!! note
        The range units can be disregarded if this class is not being used for astrodynamic transforms.

    Args:
        range: distance from the origin in **_kilometers_**
        right_ascension: right ascension in **_degrees_**
        declination: declination in **_degrees_**
    """

    range: float
    right_ascension: float
    declination: float
    def __init__(self, range: float, right_ascension: float, declination: float) -> None: ...
    def to_cartesian(self) -> CartesianVector:
        """
        Returns:
            Cartesian vector in **_kilometers_**.
        """
        ...

class CartesianVector:
    """
    Args:
        x: x coordinate
        y: y coordinate
        z: z coordinate
    """

    x: float
    y: float
    z: float
    magnitude: float
    """"""

    def __init__(self, x: float, y: float, z: float) -> None: ...
    def distance(self, other: CartesianVector) -> float:
        """
        !!! note
            Take care to use consistent units with this function.

        Returns:
            Distance between two Cartesian vectors
        """
        ...

    def to_spherical(self) -> SphericalVector:
        """
        Returns:
            Spherical representation of the point
        """
        ...

    def __add__(self, other: CartesianVector) -> CartesianVector: ...
    def __sub__(self, other: CartesianVector) -> CartesianVector: ...
    def angle(self, other: CartesianVector) -> float:
        """
        Returns:
            Angle between two Cartesian vectors in **_radians_**.
        """
        ...

class CartesianState:
    """State represented as x, y, z and vx, vy, vz in a given reference frame.

    Args:
        epoch: UTC epoch of the state
        position: Cartesian position vector in kilometers
        velocity: Cartesian velocity vector in kilometers per second
        frame: reference frame of the state
    """

    position: CartesianVector
    """Position of the state in kilometers"""

    velocity: CartesianVector
    """Velocity of the state in kilometers per second"""

    epoch: Epoch
    """UTC epoch of the state"""

    frame: ReferenceFrame
    """Current reference frame of the state"""

    def __init__(
        self, epoch: Epoch, position: CartesianVector, velocity: CartesianVector, frame: ReferenceFrame
    ) -> None: ...
    def to_keplerian(self) -> KeplerianState:
        """Convert the Cartesian state to osculating Keplerian elements"""
        ...

    def to_frame(self, frame: ReferenceFrame) -> CartesianState:
        """

        Args:
            frame: reference frame of the output state

        Returns:
            CartesianState: Cartesian state in the new frame"""
        ...

class KeplerianState:
    """Orbit represented as Keplerian elements in a given reference frame.

    Args:
        epoch: UTC epoch of the state
        elements: Keplerian elements of the state
        frame: reference frame of the state
        keplerian_type: type of the Keplerian elements
    """

    epoch: Epoch
    frame: ReferenceFrame
    type: KeplerianType
    semi_major_axis: float
    """Average distance from the central body in **_kilometers_**"""

    eccentricity: float
    """Eccentricity of the orbit"""

    inclination: float
    """Inclination of the orbit in **_degrees_**"""

    raan: float
    """Right Ascension of Ascending Node in **_degrees_**"""

    argument_of_perigee: float
    """Argument of Perigee in **_degrees_**"""

    mean_anomaly: float
    """Mean Anomaly in **_degrees_**"""

    mean_motion: float
    """Mean motion in **_revolutions per day_**"""

    apoapsis: float
    """Furthest point from the central body in **_kilometers_**"""

    periapsis: float
    """Closest point to the central body in **_kilometers_**"""

    def __init__(
        self,
        epoch: Epoch,
        elements: KeplerianElements,
        frame: ReferenceFrame,
        keplerian_type: KeplerianType,
    ) -> None: ...
    def to_cartesian(self) -> CartesianState:
        """
        Returns:
            Cartesian state in **_kilometers_** and **_kilometers per second_**.
        """
        ...

    def to_frame(self, frame: ReferenceFrame) -> KeplerianState:
        """
        Args:
            frame: reference frame of the output state

        Returns:
            Keplerian state in the new frame"""
        ...

class Ephemeris:
    def get_close_approach(
        self,
        other: Ephemeris,
        distance_threshold: float,
    ) -> CloseApproach: ...

class TopocentricElements:
    """
    Args:
        right_ascension: TEME right ascension in **_degrees_**
        declination: TEME declination in **_degrees_**
    """

    range: Optional[float]
    """Range in **_kilometers_**"""

    right_ascension: float
    declination: float
    range_rate: Optional[float]
    """Range rate in **_kilometers per second_**"""

    right_ascension_rate: Optional[float]
    """Right ascension rate in **_degrees per second**"""

    declination_rate: Optional[float]
    """Declination rate in **_degrees per second**"""

    def __init__(self, right_ascension: float, declination: float) -> None: ...
    @classmethod
    def from_j2000(cls, epoch: Epoch, ra: float, dec: float) -> TopocentricElements:
        """
        Args:
            epoch: UTC epoch of the angles
            ra: J2000 right ascension in **_degrees_**
            dec: J2000 declination in **_degrees_**
        """
        ...

class TopocentricState:
    """
    Args:
        epoch: UTC epoch of the state
        elements: TopocentricElements of the state
    """

    epoch: Epoch
    """UTC epoch of the state"""

    elements: TopocentricElements
    """Topocentric elements of the state"""

    range: Optional[float]
    """Range in **_kilometers_**"""

    right_ascension: float
    """TEME right ascension in **_degrees_**"""

    declination: float
    """TEME declination in **_degrees_**"""

    range_rate: Optional[float]
    """Range rate in **_kilometers per second_**"""

    right_ascension_rate: Optional[float]
    """Right ascension rate in **_degrees per second**"""

    declination_rate: Optional[float]
    """Declination rate in **_degrees per second**"""

    def __init__(self, epoch: Epoch, elements: TopocentricElements) -> None: ...
    @classmethod
    def from_horizon_state(cls, horizon_state: HorizonState, observer: Observatory) -> TopocentricState:
        """
        Args:
            horizon_state: HorizonState of the target
            observer: Position of the observer
        """
        ...
